#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include <stdio.h>
#include <string.h>

#include "buzzer.h"
#include "fnd.h"
#include "ir.h"
#include "key.h"
#include "led.h"
#include "var.h"


extern volatile char key_flag;
extern volatile char fnd_flag;
extern volatile unsigned char cur_temp;
extern volatile unsigned char buzzer_on, buz_timer;

volatile unsigned char mode, ir_detect;
volatile unsigned char led_pos, alarm_on;
volatile unsigned int sleep_timer;
enum {
	NO_MODE,
	HOME_MODE,
	OUT_MODE,
	SLEEP_MODE
};
enum {
	OFF,
	ON
};


void timer0_init(void)
{
	TCCR0 = 0x06;
	TCNT0 = 256 - 250;
	TIMSK |= 0x01;
}

ISR(TIMER0_OVF_vect)
{
	static unsigned char tick0;

	TCNT0 = 256 - 250;

	if( ir_detect || alarm_on ) {
		if( ++tick0 == 250 ) {
			tick0 = 0;
			led_pos = ++led_pos % 8;
			led_light(0x80 >> led_pos);
		}
	}

	if( mode == SLEEP_MODE ) {
		sleep_timer--;
		if( sleep_timer == 0 ) {
			sleep_timer = 500;
			if( cur_temp )	cur_temp--;
		}
	}
}

void mcu_init(void)
{
	buzzer_init();
	fnd_init();
	ir_init();
	key_init();
	led_init();
	var_init();

	timer0_init();

	sei();
}

void start_boiler(void)
{
	led_light(LED_ALL);
	_delay_ms(1000);
	led_light(0x00);

	fnd_flag = ON;
	mode = NO_MODE;
	ir_detect = OFF;
}

void check_temperature(void)
{
	unsigned char led;

	cur_temp = (unsigned char)(var_read() / 12.7);

	//////////////////////////////////////////////////////////////////////////
	// [ 3] µ   ︲ 
	// (9) 䱸 (5)    ۵ǰ Ͻÿ.
	//////////////////////////////////////////////////////////////////////////
	if( cur_temp >= 40 ) {
		buzzer_on = ON;

		if( cur_temp >= 40 && cur_temp < 50 )		buz_timer = 5 * 10;
		else if( cur_temp >= 50 && cur_temp < 60 )	buz_timer = 35;
		else if( cur_temp >= 60 && cur_temp < 70 )	buz_timer = 2 * 10;
		else if( cur_temp >= 70 )				buz_timer = 1 * 10;
	}
	else	buzzer_on = OFF;

	//////////////////////////////////////////////////////////////////////////
	// [ 1] µ   ǥ  
	// (3) VR1 µ ϴ ̴. HOME SW  VR1 ϸ
	//    µ FND ǥõǰ Ͻÿ
	//    (, µ 00~80  ǥõȴ. FND1=10 ڸ, FND2=1 ڸ)
	//////////////////////////////////////////////////////////////////////////
	led = 0;
	for( int i = 0; i < (cur_temp / 10); i++ ) {
		led |= (0x80 >> i);
	}
	led_light(led);
}

int main(void)
{
	unsigned char key = 0;

	mcu_init();
	start_boiler();

	while( 1 ) {
		key = getkey(key);
		if( key_flag ) {
			switch( key ) {
				case KEY_HOME :
					if( mode != HOME_MODE ) {
						if( ir_detect ) {
							led_light(0x00);
							ir_detect = OFF;
						}

						alarm_on = OFF;
						buzzer_on = OFF;

						mode = HOME_MODE;
						gled_light(OFF);
					}
					break;

				case KEY_OUT :
					if( mode != OUT_MODE ) {
						if( !ir_detect ) {
							mode = OUT_MODE;
							gled_light(ON);
						}

						alarm_on = OFF;
						buzzer_on = OFF;

					}
					break;

				case KEY_SLEEP :
					if( mode != SLEEP_MODE ) {
						if( !ir_detect ) {
							sleep_timer = 500;
							cur_temp = 9;
							alarm_on = OFF;

							gled_light(OFF);
							led_light(OFF);

							mode = SLEEP_MODE;
						}
					}
					break;
			}
		}

		if( mode == HOME_MODE )		check_temperature();
		else if( mode == OUT_MODE ) {
			if( !ir_detect ) {
				check_temperature();
				////////////////////////////////////////////////////////
				// [ 2] ܼ  
				// () ܼ  ߱ο  ̿ ü 
				//	  µ  , FND 88, LED9~2
				//	  ϳ 1   ҵ ݺϰ Ͻÿ. 
				//        1  ︮ Ͻÿ.
				////////////////////////////////////////////////////////
				if( ir_check() ) {
					buzzer_on = ON;
					buz_timer = 5 * 10;
					cur_temp = 88;

					led_pos = 0;
					led_light(0x80);

					ir_detect = ON;
				}
			}
		}
		else if( mode == SLEEP_MODE ) {
			if( cur_temp == 0 ) {
				if( !alarm_on ) {
					buzzer_on = ON;
					buz_timer = 5 * 10;

					led_pos = 0;
					led_light(0x80);

					alarm_on = ON;
				}
			}
		}
	}

	return 0;
}
